hirom   ; don't change this
header  ; comment out if your rom has no header

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Description of work
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; All of this work is inline. Meaning that while my changes may have created free space, they did not 
;   move any functionality for anything other than Blitz... nor did it use any existing free space in the ROM
; The only exception to this is that I cannibalized index #$15 in the pointer table at C1/6414 because it was
;   unused and I needed it. I also repointed a pointer in non-battle menu code for Dance as it uses a shared
;	function with Blitz that needed to be relocated

; Needed parts to create this patch:
; Create Battle Menu generator function
; Create Battle Menu loop function
; Create Battle Menu select item function
; Create Battle Menu layout data
; Create Blitz ability target data
; Alter Non-battle Menu text function

; This work involves a lot of pointer changes... Much of it is based on corresponding Dance and Magitek menu code
;   Here is a break down of what some of that means...

; There is a pointer table at C1/552C which calls menu building, and menu item selection code for battle commands
;   It uses the value of $7BC2 as its index. Values indexed by #$3B, #$3C, and #$3D are used by Blitz. They handle 
;   menu drawing, looping, and menu item selection respectively. 
; $7BC2 is set up by looking at what is in the Accumulator when calling the function at $5A2C. The function looks at
;   the 4 bytes located at ($C2E348 + Accumulator * 4). Blitz uses #$11 in the Accumulator,
;   As such I needed to change the data block of 4 bytes at C2/E38C.

; There is another pionter table at C1/4517 which handles menu layout functionality. It uses the value of $7BF0 as
;   its index. Values indexed by #$27 and #$28 are the values associated with Blitz.

; There is another pointer table at C1/6414. This table controls what function gets called to load menu text based on 
;   the value of $2C when invoked from C1/6411, and gets referenced by the data bytes described by this web page:
;   http://datacrystal.romhacking.net/wiki/Final_Fantasy_VI:Combat_Menu_Format
; This is no Blitz reference on that web page, so I had to use one of the "unused" indexes... I chose to use #$15.
;   This does not seem to cause any issue, but it might so it is worth noting. I also had to use some of the free
;   space I created to store menu format bytes for Blitz since those also did not exist.

; Last there is a pointer table at C1/7CDE which calls the build menu funciton for battle commands
;   Value indexed by #$0A is used by Blitz (#$00 is used by Fight, #$1D by Magitek, etc... so this should seem familiar)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Space freed by this patch
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;	C1/5755 - C1/575E :  9 bytes : Dead code at old "3B" fucntion location
;	C1/575F - C1/5768 :  9 bytes : Dead code at old "3C" function location
;	C1/7083 - C1/7095 : 18 bytes : Dead code at end of old "3D" function location
;	C3/5693 - C3/56BB : 40 bytes : Dead code at end of old Blitz non-battle menu display
;	C4/7A55 - C4/7A9F : 74 bytes : Useless data at end of old Blitz button combo data
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;       Assembly code follows       ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;  Pointer changes
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; change blitz $7BF0 function pointers ; handles/alters spacing on menu that I can tell...
; changed to mimic Magitek / Dance / etc that use these same functions
org $C14565                                      ; ORG CHANGE

DW #$5088
DW #$50B6

; change blitz pointers for function $5A2C
org $C155A2                                      ; ORG CHANGE

DW Blitz3B	; menu pointer
DW Blitz3C	; loop pointer
DW Blitz3D  ; use menu item pointer

; change blitz menu text pointer
; BUG RISK this overrides whatever was using 15, which I don't see anywhere but...
; C1/643E:	4767    	(15)			(exits)
org $C1643E                                      ; ORG CHANGE

DW GetText

; change blitz menu pointer
org $C17CFD                                      ; ORG CHANGE

DW Blitz3B	; another reference to menu pointer

; Pointer for menu item selectiong code
org $C2E38C                                      ; ORG CHANGE

DW #$3D01 ; make point to 3D

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;  Battle Menu changes
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;change blitz functions
org $C16F6A                                      ; ORG CHANGE

;;;;;;;;;;;;;;; Menu Builder code ;;;;;;;;;;;;;;;;;;

Blitz3B:										 
LDA #$3B			; set up self reference
STA $7BC2			; store self reference
LDA $7BA5
BMI StartMenuBuild3B ; branch if minus ?
JSR $5A17			 ; common routine
STZ $7BA6
LDA #$80
STA $7BA5

StartMenuBuild3B:
LDA $7BA6
JSR MenuBuild3B
JSR $5729			; common routine
BCC Return3B
LDA #$27			; set up menu layout handler pointer
STA $7BF0			; store it
LDA #$11			; set up menu item data block location (data block contains menu select item function pointer)
JMP $5A2C			; common routine ... stores bytes of item data block into $7BC2 so...

Return3B:
RTS

MenuBuild3B:
PHY
ASL A
TAY
TDC
TAX

LoadCyc3B:
LDA MenuData,X		; format for drawing blitz menu (13 bytes long)
STA $5755,X
INX
CPX #$000D			; did we get 13 bytes?
BNE LoadCyc3B		; branch if not
TYA
TAX
JSR GetBlitzIndex	; (Left column Blitz attacks)
STA $575A
PHX					; save X
INX					; increment it by 1 so we get blitz on the Right side
JSR GetBlitzIndex	; (Right column Blitz attacks)
PLX					; restore X so we don't mess up other functions in this interation
STA $5760
JSR $4E07			; common function - used to set up menu format??
JSR $63AB			; common function - used to get text to display based on menu format... #$15 for blitz...
PLY
RTS

;;;;;;;;;;;;;;; Menu Item Selection code ;;;;;;;;;;;;;;;;;;

Blitz3D:								
STZ $88E3			
LDA $7BCB
BEQ LoadComm3D
LDA #$3C 
STA $7BC2			; (queue up to execute Menu Looping Code)
RTS

LoadComm3D: 
LDY $62CA
LDA $893F,Y
STA $36
LDA $8943,Y
STA $37
STZ $38
LDA #$01
STA $39
JSR $80DB			; common function
LDA $3C
BNE LoadContrl3D
LDA $36
STA $893F,Y
LDA $37
STA $8943,Y

LoadContrl3D:
LDA $09				;  (load controller 1)
BPL LoadOptn3D		;  (branch if not pressing B)
INC $96
JSR Blitz3C
RTS

;Load command options ;;;;;;;;;;;;;;;;;;
LoadOptn3D:
STY $36
LDA $04				;   (load shared controller byte)
BPL SkipMid3D		;   (branch if nobody is pressing A)
JSR $8491			; not common function, but probably don't want to replicate; touches carry flag though, be careful with compares...
JSR GetBlitzIndex
CMP #$FF
BEQ SlotEmpty3D    ;  (branch if slot is empty, note that this will never be taken)
INC $96
STA $7A85
LDY $36
LDA TargetData,X   ;    (Targeting byte for Blitz commands)
STA $7A84			;  (save as character targeting byte)
JMP $7795			;  common function ; jump into fight routine

SlotEmpty3D:
INC $95

SkipMid3D:
LDY $36
LDA $893F,Y
TAX 
LDA $C18289,X   ;  load finger positioning for magitek menu
STA $88E4
LDA $8943,Y
TAX 
LDA $C1828D,X  ; load something... out of a data table? Vertical finger positioning?
STA $88E5
INC $88E3
RTS

;;;;;;;;;;;;;;; Menu Item Text code ;;;;;;;;;;;;;;;;;;
GetText:
JSR $63F7		;	(from C1/6411, X = 15) 
LDA ($4F)
STA $2C			; storing if the attack is available or not?
CMP #$FF
BNE TextGetName
LDA #$0A
JMP $66A5		; common function?

TextGetName:
;STA $2C
LDA #$0A		;	(10 = Blitz attack name length)
STA $2E
STA $40
JSR $18B0		; common function
LDX $30

TextLoadName:
LDA $E6F831,X	;	(Loads Blitz attack name X)
JSR $63FE		; common function?
INX 
DEC $40
BNE TextLoadName
RTS

;;;;;;;;;;;;;;; Menu Looping code ;;;;;;;;;;;;;;;;;;
Blitz3C:										 
LDA #$28	; set up menu layout
STA $7BF0
TDC			; set up next menu call
JMP $5A2C	; common function

;;;;;;;;;;;;;;; Menu Item Retrieval code ;;;;;;;;;;;;;;;;;;
;using $1D28 (bit mask of known blitzes)
; given X, load either X or FF, depending on bit mask
GetBlitzIndex:
PHY
TXY						; set Y = X
LDA #$01				; initialize to check first bit (check if Pummel is known)
DEY						
BMI GetBlitzCheckMask	; if Y was 0, we are done. Otherwise rotate bits to check appropriate Blitz

GetBlitzRollBits:
ASL						; shift bits left, could use ROL but that seemed to cause issues
DEY
BPL GetBlitzRollBits	; If Y not yet 0, we haven't made the right bit mask for the Blitz, keep rotating

GetBlitzCheckMask:
BIT $1D28				; compare known Blitzes mask to the ability mask we just made
BNE GetBlitzValidRet	; if its valid return X (the index of the ability)
LDA #$FF				; Otherwise return the "null" ability
PLY
RTS

GetBlitzValidRet:
TXA
PLY
RTS

; Free space

;@ C1/7095 is where blitz originally ended so the above code must be shorter than 300 bytes or stuff breaks!

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;  Non-Battle Menu Text changes
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Previously the non-battle Blitz menu showed button combo to push, we want to display the Blitz name instead

org $C355D4                                      ; ORG CHANGE

JSR $6A15			; common function
JSR DrawInPos
LDA #$2C			;  (blue)
STA $29				;  (set text color to blue)
LDY #$5CB0			;  (Address at which you find the word "Blitz")
JSR $02F9			; common function
JSR $61AC			; common function
JMP $0F4D			; common function

DrawInPos:
JSR LoadLearned
LDA #$20			; (white)
STA $29				; (set text color to white)
JSR $83F7			; common function
STZ $E5
INC $E6
LDY #$0004

DrawNextPos:
PHY
JSR DrawBlitzLinePos    ;   (set both positions for blitz combination display)
LDA $E6
INC A
INC A
INC A
INC A					;  (two lines for next blitz sequence)
AND #$1F
STA $E6
PLY
DEY
BNE DrawNextPos
RTS

DrawBlitzLinePos:
JSR GetAddr			; setup Blitz text location and size
LDX #$0004			; (X position of left half of Blitz)
JSR SetCharOrBlank
INC $E5
JSR GetAddr			; setup Blitz text location and size
LDX #$0012			; (X position of right half of Blitz)
JSR SetCharOrBlank
INC $E5
RTS 

; setup Blitz text location and size
GetAddr:
LDY #$000A		; (Blitz name length)
STY $EB
LDY #$F831		; (Blitz address)
STY $EF
LDA #$E6		; (Blitz bank)    ; So this will look in E6/F831 + X * $000A for the blitz text
STA $F1
RTS 

LoadLearned:
LDA $1D28		;  (load Blitz's learned)
BRA SkipDance

LoadDncLrned:   ; Called from Dance menu code
LDA $1D4C		;  (load Dance's learned)

SkipDance:
LDX #$9D89
STX $2181
LDX $00
LDY #$0008

RotateRight:
ROR A
PHA
BCC DrawSpace
TXA
STA $2180
BRA DrawChar

DrawSpace:
LDA #$FF
STA $2180		;   (draw a space)

DrawChar:
INX
PLA
DEY
BNE RotateRight
RTS
 
SetCharOrBlank:
LDA $E6
JSR $809F		; common function
REP #$20		;   (16 bit memory/accum.)
TXA
STA $7E9E89
SEP #$20		;   (8 bit memory/accum.)
TDC
LDA $E5
TAX
LDA $7E9D89,X
CMP #$FF
BEQ DrawEmpty
JSR $8467		; common function (draw the current blitz name) ; uses values of #$EB, #$EF, and #$F1 to do text lookup
JMP $7FD9		; common function

DrawEmpty:
LDY #$0008
LDX #$9E8B
STX $2181
LDA #$FF			;   (A already holds #$FF)

ContDrawEmpty:
STA $2180		   ;   (draw a blank space)
DEY
BNE ContDrawEmpty   ;   (branch if haven't drawn 8 blank spaces)
STZ $2180		    ;   (end this string)
JMP $7FD9

; Free space

; still need to update dance pointer since it shares a function with blitz
org $C3578A                                      ; ORG CHANGE

JSR LoadDncLrned

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;  Blitz Data block changes
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Previously this contained the button combonations used to match against player button presses
;   as well as display the button combo in the non-battle menu. Since we aren't doing either
;   of those anymore we can use this as free space.
; We will store Blitz targeting information (self or random enemy as appropriate)
; We will also store Blitz menu layout bytes
; These data blocks are used in the Menu Building and Menu Item Selection code

org $C47A40                                      ; ORG CHANGE

;; see http://datacrystal.romhacking.net/wiki/Final_Fantasy_VI:Combat_Menu_Format for what this means
MenuData:
DD #$21040405
DD #$03050015   ; swap word retrieval code to use 0x15 pointer... Not sure what used this originally though!! BUG RISK
DD #$00152104   ; swap word retrieval code to use 0x15 pointer... Not sure what used this originally though!! BUG RISK
DB #$00

TargetData:
DD #$6A6A6A6A	; 6A means random / all enemies
DD #$6A026A02	; 02 means self only

; Free space